#include "T6963.h"
#include <Wire.h>          // I2C Bus Library DS1307,DS3231
#include <EEPROM.h>        // To store settings
#include "EEPROMAnything.h" // To store settings
#include <RTClib.h>        // Standard RTC library
#include "Harddisc__30.h" //Digital Clock Font
#include "GLCD_Clock.h"   //Analog Clock Picture
#include <avr/pgmspace.h>

#define DS3231_I2C_ADDR             0x68 // Used to get temperature from internal DS3231 sensor
#define DS3231_TEMPERATURE_ADDR     0x11 // Used to get temperature from internal DS3231 sensor

unsigned long previousMillis = 0;    
const long interval = 1000;
  

int pumpPin = 10;
int blPin = 13;
int hourButton = 11;
int durationButton = 12;

T6963 LCD(240,64,6,32);// 240x64 Pixel and 6x8 Font

RTC_DS1307 ds1307; 
const int DS1307 = 0x68;      

byte decToBcd(byte val) {
  return ((val/10*16) + (val%10));
}
byte bcdToDec(byte val) {
  return ((val/16*10) + (val%16));
}

byte second = 0;
byte minute = 0;
byte hour = 0;
byte weekday = 0;
byte monthday = 0;
byte month = 0;
byte year = 0;

int waterHour = 8;
long int waterDuration = 0;
int blTime = 12;

char tVal[6];
char wHour[2];
char wDur[4];

float DS3231_get_treg()
{
    short rv;  
    
    Wire.beginTransmission(DS3231_I2C_ADDR);
    Wire.write(DS3231_TEMPERATURE_ADDR);
    Wire.endTransmission();

    Wire.requestFrom(DS3231_I2C_ADDR, 2);
    rv = Wire.read() << 8;
    rv |= Wire.read();
  return(rv/256.);
}

// Clock init. 
char strclock[26];
const char* days[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
const char* months[] = { 0, "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };// Monat
int sec, pcount, state=0, HR_circle, MIN_circle, SEC_circle;

// Set Realtime Clock with built in Serial Monitor

// Dayweek line
void dayofweek(int x, int y) //Day of the Week x,y
      {
      DateTime now = ds1307.now();
      int offset=3;    //Offset Position Cursor
      LCD.TextGoTo(x,y);
      LCD.writeString("Su Mo Tu We Th Fr Sa");
      int dow=(now.dayOfTheWeek())*offset;
      LCD.drawrectbyte((x+dow)*LCD.getFw(),y,8,2,0b111111); // draw dayweek Cursor
      }         
      
// Digital Clock
void digital_clock(int x, int y, int TimeZone) //Position x, Position y, Set TimeZone
      {
      DateTime now = ds1307.now();
      sprintf(strclock, "%2.2d:%2.2d:%2.2d", (now.hour())+TimeZone, (now.minute()), (now.second())); 
      LCD.glcd_print2_P( x, y, (strclock), &Harddisc__30,0);
      }   
            
//  Day Month Year line     
void daymonthyear(int x, int y) //Day Month Year x,y
      {
      DateTime now = ds1307.now();
      int size = 0 ;  
      sprintf(strclock, "%2.2d.%s.%4.4d", (now.day()), months[(now.month())],(now.year())); 
      size = strlen(strclock);
      LCD.TextGoTo(x-size,y);
      LCD.writeString(strclock);
      } 

/*
	Draw clock hands
	Parameter: x, y: center radius start angle, end angle. 0 = Top, 90=right clockwise, color 1=on 0=off
*/
void analog_clock(int x, int y, int radius, int angle_start, int angle_end, int color)
{
	int	x_circle, y_circle;
	float corner;
	#define DEGREE 2*3.14159265/360	
	
	for (corner = (float)angle_start; corner <= angle_end; corner +=1) // Calculate clock hands
	    {
	    x_circle = (int) (sin (corner * DEGREE) * radius);
	    y_circle = (int) (cos (corner * DEGREE) * radius);
	    LCD.createLine(32,32, x + x_circle, y - y_circle, color); //draw clock hands
	    }
}

void clock_main()// full screen clock main
      {
      DateTime now = ds1307.now();
      dayofweek(20,0); // Position x,y
      digital_clock(90,12,0); // Position x,y,TimeZone
      daymonthyear(40, 7);  // Position x,y
      
       if((now.second()) == 00) 
         {
         analog_clock(32,32,18,HR_circle,HR_circle,0 );   //clears Hour clock hands if Second is 0
         analog_clock(32,32,24,MIN_circle,MIN_circle,0 ); //clears Minute clock hands if Second is 0
         }
           analog_clock(32,32,24,SEC_circle,SEC_circle,0 ); //clears Second clock hands every second  
   
   
       HR_circle = (now.hour());
    
       if(HR_circle >= 12) HR_circle-=12;
       
       HR_circle = ((HR_circle * 30)+(now.minute())/2); // Hour 30 Degrees for next Position and every minute 6 degrees forward
       MIN_circle = (now.minute())*6;  // Minute 6 Degrees for next Position
       SEC_circle = (now.second())*6;  // Second 6 Degrees for next Position
       
       // draw clock hands
       analog_clock(32, 32, 24, MIN_circle, MIN_circle, 1); //draw Minute clock hands
       analog_clock(32, 32, 18, HR_circle, HR_circle, 1); //draw Hour clock hands
       analog_clock(32, 32, 24, SEC_circle, SEC_circle, 1); //draw Second clock hands
       
}

// This set of codes is allows input of data
void setTime() {
  Serial.print(F("Please enter the current year, 00-99. - "));
  year = readByte();
  Serial.println(year);
  Serial.print(F("Please enter the current month, 1-12. - "));
  month = readByte();
  Serial.println(months[month-1]);
  Serial.print(F("Please enter the current day of the month, 1-31. - "));
  monthday = readByte();
  Serial.println(monthday);
  Serial.println(F("Please enter the current day of the week, 1-7."));
  Serial.print(F("1 Sun | 2 Mon | 3 Tues | 4 Weds | 5 Thu | 6 Fri | 7 Sat - "));
  weekday = readByte();
  Serial.println(days[weekday-1]);
  Serial.print(F("Please enter the current hour in 24hr format, 0-23. - "));
  hour = readByte();
  Serial.println(hour);
  Serial.print(F("Please enter the current minute, 0-59. - "));
  minute = readByte();
  Serial.println(minute);
  second = 0;
  Serial.println(F("The data has been entered."));
 
  // The following codes transmits the data to the RTC
  Wire.beginTransmission(DS1307);
  Wire.write(byte(0));
  Wire.write(decToBcd(second));
  Wire.write(decToBcd(minute));
  Wire.write(decToBcd(hour));
  Wire.write(decToBcd(weekday));
  Wire.write(decToBcd(monthday));
  Wire.write(decToBcd(month));
  Wire.write(decToBcd(year));
  Wire.write(byte(0));
  Wire.endTransmission();
  // Ends transmission of data
}


byte readByte() {
  while (!Serial.available()) delay(10);
  byte reading = 0;
  byte incomingByte = Serial.read();
  while (incomingByte != '\n') {
    if (incomingByte >= '0' && incomingByte <= '9')
      reading = reading * 10 + (incomingByte - '0');
    else;
    incomingByte = Serial.read();
  }
  Serial.flush();
  return reading;
}

void printTime() {
  char buffer[3];
  const char* AMPM = 0;
  readTime();
  Serial.print(days[weekday-1]);
  Serial.print(" ");
  Serial.print(months[month-1]);
  Serial.print(" ");
  Serial.print(monthday);
  Serial.print(", 20");
  Serial.print(year);
  Serial.print(" ");
  if (hour > 12) {
    hour -= 12;
    AMPM = " PM";
  }
  else AMPM = " AM";
  Serial.print(hour);
  Serial.print(":");
  sprintf(buffer, "%02d", minute);
  Serial.print(buffer);
  Serial.println(AMPM);
}

void readTime() {
  Wire.beginTransmission(DS1307);
  Wire.write(byte(0));
  Wire.endTransmission();
  Wire.requestFrom(DS1307, 7);
  second = bcdToDec(Wire.read());
  minute = bcdToDec(Wire.read());
  hour = bcdToDec(Wire.read());
  weekday = bcdToDec(Wire.read());
  monthday = bcdToDec(Wire.read());
  month = bcdToDec(Wire.read());
  year = bcdToDec(Wire.read());
}

void updateTimer()
{
      dtostrf(waterHour,0,0,wHour);
      dtostrf(waterDuration,0,0,wDur);
      LCD.TextGoTo(10,0);
      LCD.writeString("On:");
      LCD.TextGoTo(13,0);
      if (waterHour <10)
      LCD.writeString("0");
      LCD.writeString(wHour);
      LCD.TextGoTo(15,0);
      LCD.writeString(":00");
      LCD.TextGoTo(10,1);
      LCD.writeString("For:");
      LCD.TextGoTo(14,1);
      LCD.writeString(wDur);
      LCD.writeString("S  ");
}

void runPump()
{
   digitalWrite (pumpPin, HIGH);
   digitalWrite (blPin, HIGH);
   
   while (waterDuration >=1)
   {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval) 
    {
    previousMillis = currentMillis;
    clock_main();
    waterDuration--; 
    }
   
   }

   if (waterDuration == 00)
   {
   digitalWrite (pumpPin, LOW);
   digitalWrite (blPin, LOW); 
   waterDuration = EEPROM_readAnything(10,waterDuration);
   }
}

void backlightControl()
{

  digitalWrite (blPin, HIGH);
  while (blTime >=1)
  {
  unsigned long currentMillis = millis();
   if (currentMillis - previousMillis >= interval) 
    {
    previousMillis = currentMillis;
    clock_main();
    blTime--;
    }
  }

    if (blTime == 00)
    {
    digitalWrite (blPin, LOW);
    blTime = 12;
    }
}

void monitorButtons()
{
      int hourval = 0;
      hourval = digitalRead(hourButton);
      if (hourval == LOW)
      {
      waterHour++;
      digitalWrite (blPin, HIGH);
      delay(10);
      EEPROM_writeAnything(0,waterHour);  
      if (waterHour == 24)
      waterHour = 0;
      }
     
      int durval = 0;
      durval = digitalRead(durationButton);
      if (durval == LOW)
      {
      waterDuration++;
      digitalWrite (blPin, HIGH);
      delay(10);
      EEPROM_writeAnything(10,waterDuration);  
      if (waterDuration == 360)
      waterDuration = 0;
      }

}

void setup() {
 Serial.begin(9600); 
 ds1307.begin();
 pinMode(pumpPin, OUTPUT);
 pinMode(blPin, OUTPUT);
 pinMode(hourButton, INPUT_PULLUP);
 pinMode(durationButton, INPUT_PULLUP);
 digitalWrite(pumpPin, LOW);
 digitalWrite(blPin, LOW); 
 waterHour = EEPROM_readAnything(0,waterHour);
 waterDuration = EEPROM_readAnything(10,waterDuration);
 //EEPROM_writeAnything(0,waterHour); // comment these two back in for the first time & uncomment above two, run then change back, re-upload
 //EEPROM_writeAnything(10,waterDuration);
 

 delay(500); // This delay allows the MCU to read the current date and time.
 
  Serial.print(F("The current date and time is: "));
  printTime();
  Serial.println(F("Please change to newline ending the settings on the lower right of the Serial Monitor"));
  Serial.println(F("Would you like to set the date and time now? Y/N")); 
    
  if (Serial.available()>=0) delay(5000); // wait 5 seconds for user input otherwise continue boot
  
  if (Serial.read() == 'y' || Serial.read() == 'Y')  
    
  // This set of functions allows the user to change the date and time
  {
    Serial.read();
    setTime();
    Serial.print(F("The current date and time is now: "));
    printTime();
  }

  Serial.println(F("Done."));
  LCD.Initialize();
  LCD.setMode('x','I'); //Mode Set for Reverse Text T6963
  LCD.drawPic( 0, 0, GLCD_Clockbmp ,GLCD_ClockHEIGHT, GLCD_ClockBYTEWIDTH ); // draw Clock Wallpaper
  
}


void loop()
{
      DateTime now = ds1307.now();            
      // draw Temp line
      float ds3231temp = DS3231_get_treg();
      dtostrf(ds3231temp-3,4,2,tVal);    // convert temperature value to xx.xx output format               
      LCD.TextGoTo(10,7);
      LCD.writeString("Temp:");
      LCD.writeString(tVal);
      LCD.writeString("C ");

    if (waterHour == now.hour() && (now.minute() == 00) && (now.second() == 00)) // Run pump at scheduled time
    {
      runPump();
      clock_main();
    }
	
	else if (waterHour+8 == now.hour() && (now.minute() == 00) && (now.second() == 00)) // run pump 8 hours after scheduled time
	{
	   runPump();
	   clock_main();
	}

      else
     {
      monitorButtons();
      updateTimer();
      clock_main(); 
     }   

      if ( (now.second() % 30) == 0)
      digitalWrite (blPin, LOW);
     
}
